package org.nakedobjects.nos.store.file;

import java.util.Enumeration;
import java.util.Hashtable;

import org.nakedobjects.noa.NakedObjectRuntimeException;
import org.nakedobjects.noa.adapter.NakedCollection;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.persist.SerialOid;


/**
 * A logical collection of elements of a specified type
 */
public class ObjectData extends Data {
    private Hashtable fields;

    public ObjectData(final NakedObjectSpecification type, final SerialOid oid, final FileVersion version) {
        super(type, oid, version);
        fields = new Hashtable();
    }

    public String toString() {
        return "ObjectData[type=" + getClassName() + ",oid=" + getOid() + ",fields=" + fields + "]";
    }

    public void set(final String fieldId, final Object oid) {
        if (oid == null) {
            fields.remove(fieldId);
        } else {
            fields.put(fieldId, oid);
        }
    }

    void saveValue(final String fieldId, final boolean isEmpty, final String encodedString) {
        if (isEmpty) {
            fields.remove(fieldId);
        } else {
            fields.put(fieldId, encodedString);
        }
    }

    public void set(final String fieldId, final String value) {
        fields.put(fieldId, value);
    }

    public Object get(final String fieldId) {
        return fields.get(fieldId);
    }

    public String value(final String fieldId) {
        return (String) get(fieldId);
    }

    public String id(final String fieldId) {
        Object field = get(fieldId);
        return field == null ? null : "" + ((SerialOid) field).getSerialNo();
    }

    void initCollection(final SerialOid collectionOid, final String fieldId) {
        fields.put(fieldId, new ReferenceVector(collectionOid));
    }

    void addElement(final String fieldId, final SerialOid elementOid) {
        if (!fields.containsKey(fieldId)) {
            throw new NakedObjectRuntimeException("Field " + fieldId + " not found  in hashtable");
        }

        ReferenceVector v = (ReferenceVector) fields.get(fieldId);
        v.add(elementOid);
    }

    ReferenceVector elements(final String fieldId) {
        return (ReferenceVector) fields.get(fieldId);
    }

    public Enumeration fields() {
        return fields.keys();
    }

    void addAssociation(final NakedObject fieldContent, final String fieldId, final boolean ensurePersistent) {
        boolean notAlreadyPersistent = fieldContent != null && fieldContent.getOid().isTransient();
        if (ensurePersistent && notAlreadyPersistent) {
            throw new IllegalStateException("Cannot save an object that is not persistent: " + fieldContent);
        }
        // LOG.debug("adding reference field " + fieldId +" " + fieldContent);
        set(fieldId, fieldContent == null ? null : fieldContent.getOid());
    }

    void addInternalCollection(final NakedCollection collection, final String fieldId, final boolean ensurePersistent) {
        /*
         * if (ensurePersistent && collection != null && collection.getOid() == null) { throw new
         * IllegalStateException("Cannot save a collection that is not persistent: " + collection); }
         */
        SerialOid coid = (SerialOid) collection.getOid();
        // Enumeration e = collection.elements();

        initCollection(coid, fieldId);

        int size = collection.size();

        Enumeration elements = collection.elements();
        while (elements.hasMoreElements()) {
            NakedObject element = (NakedObject) elements.nextElement();

            // LOG.debug("adding element to internal collection field " + fieldId +" " + element);
            Object elementOid = element.getOid();
            if (elementOid == null) {
                throw new IllegalStateException("Element is not persistent " + element);
            }

            addElement(fieldId, (SerialOid) elementOid);
        }

        /*
         * while (e.hasMoreElements()) { NakedObject element = (NakedObject) e.nextElement(); //
         * LOG.debug("adding element to internal collection field " + fieldId +" " + element); Object
         * elementOid = element.getOid(); if (elementOid == null) { throw new IllegalStateException("Element
         * is not persistent "+element); }
         * 
         * addElement(fieldId, (SimpleOid) elementOid); }
         */}
}
// Copyright (c) Naked Objects Group Ltd.
